#!/bin/bash

# Version 1.1.2
# El Capitan, Sierra

# What it does:

# Exports contents of an Apple Mail mailbox as individual emlx files to a local
# destination. Exported messages will be time-stamped and renamed according to
# their spotlight metadata.

# Usage:

# 1) In Apple Mail create a local ('on my Mac') mailbox and name it "Export".
# 2) Copy or move the messages you want to export into that mailbox.
# 3) Run this script.
# 4) You should find the exported and renamed files in a folder "Exported Mails"
#    on your Desktop.

# Note: Set your preferred time-stamp format, naming scheme, etc. by
# un/commenting the corresponding lines in the script below.


## Script

## Define source and target directories
# macOS 10.12
# It may look like this (find out the correct ID of the folder on your system):
# sourceDir=~/Library/Mail/V4/0CF67405-6830-4A6B-B162-8934DE09F521/Export.mbox
# OS X 10.11
sourceDir=~/Library/Mail/V3/Mailboxes/Export.mbox
# For older systems use:
# sourceDir=~/Library/Mail/V2/Mailboxes/Export.mbox
expDir=~/Desktop/Exported\ Mails

## Make target directory if not already present
mkdir "$expDir"

## Copy mail messages from source mailbox to target directory
find "$sourceDir" -name '*.emlx' -exec ditto {} "$expDir" \;

## Index target directory
mdimport "$expDir"
sleep 1

## Rename exported mails by Spotlight data
for f in "$expDir"/*.emlx
  do

    ## Get message author's name

    aut=$(mdls -name kMDItemAuthors -raw "$f" |
    head -n 4 | tr -ds '\n"()' ' ' |
    sed '
    s/^ //
    s/\\U[0-9a-f]\{4\}//g')

    ## Or: author's address:

    # aut=$(mdls -name kMDItemAuthorEmailAddresses -raw "$f" |
    # head -n 4 | tr -ds '\n"()' ' ' | sed 's/^ //')


    ## Get addressee names

    # adr=$(mdls -name kMDItemRecipients -raw "$f" |
    # head -n 4 | tr -ds '\n"()' ' ' |
    # sed '
    # s/^ //
    # s/\\U[0-9a-f]\{4\}//g' |
    # head -c 40)

    ## Or: addressee addresses

    # adr=$(mdls -name kMDItemRecipientEmailAddresses -raw "$f" |
    # head -n 4 | tr -d '[:space:]"()' | head -c 40)


    ## Get message subject

    suj=$(mdls -name kMDItemSubject -raw "$f" | head -c 80 |
    sed 's/\//∕/g')


    ## Content creation date to time stamp

    ccd=$(mdls -name kMDItemContentCreationDate -raw "$f")

    ## POSIX time (complete)
    # ccd=$(date -j -u -f '%Y-%m-%d %H:%M:%S %z' "$ccd" +"%s")

    ## UTC (basic, truncated year)
    # ccd=$(date -j -u -f '%Y-%m-%d %H:%M:%S %z' "$ccd" +"%y%m%dT%H%M%SZ")

    ## Local time (basic, truncated year, w/o time zone)
    # ccd=$(date -j -f '%Y-%m-%d %H:%M:%S %z' "$ccd" +"%y%m%dT%H%M%S")

    ## Local time (basic, truncated year/time, w/o time zone)
    ccd=$(date -j -f '%Y-%m-%d %H:%M:%S %z' "$ccd" +"%y%m%dT%H%M")

    ## Local time (truncated year, w/ short time zone)
    # ccd=$(date -j -f '%Y-%m-%d %H:%M:%S %z' "$ccd" +"%y%m%dT%H%M%S%z" |
    #   head -c 16)


    ## Final naming of exported messages

    # msgName="$ccd [$aut] $suj"
    msgName="$ccd $suj [$aut]"
    # msgName="$ccd [$aut → $adr] $suj"
    # msgName="$ccd [$aut - $adr] $suj"
    # msgName="$ccd $suj [$aut - $adr]"
    # msgName="$suj [$aut - $adr - $ccd]"

    mv "$f" "${f%/*}"/"$msgName"."${f#*.}"

    aCounter=$[aCounter + 1]

  done

  afplay '/System/Library/Sounds/Glass.aiff'
  osascript -e 'display notification with title "'"$aCounter"'" & " mails exported"'

  echo "Done"
  exit 0
